import * as tslib_1 from "tslib";
/** @jsx jsx */
import { jsx } from "@emotion/core";
import * as React from "react";
import { animated, useSpring } from "react-spring";
import PropTypes from "prop-types";
import { useMeasure } from "./Hooks/use-measure";
import { useUid } from "./Hooks/use-uid";
export function useCollapse(defaultShow, press) {
    if (defaultShow === void 0) { defaultShow = false; }
    if (press === void 0) { press = false; }
    var _a = React.useState(defaultShow), show = _a[0], setShow = _a[1];
    var id = useUid();
    function onClick() {
        setShow(!show);
    }
    return {
        show: show,
        setShow: setShow,
        id: id,
        buttonProps: {
            onClick: press ? undefined : onClick,
            onPress: press ? onClick : undefined,
            "aria-controls": id,
            "aria-expanded": show ? true : false
        },
        collapseProps: {
            id: id,
            show: show
        }
    };
}
/**
 * Hide and reveal content with an animation. Supports dynamic
 * heights.
 */
export var Collapse = function (_a) {
    var _b;
    var children = _a.children, id = _a.id, show = _a.show, divStyle = _a.divStyle, _c = _a.noAnimated, noAnimated = _c === void 0 ? false : _c, other = tslib_1.__rest(_a, ["children", "id", "show", "divStyle", "noAnimated"]);
    var ref = React.useRef(null);
    var bounds = useMeasure(ref).bounds;
    var prevShow = usePrevious(show);
    //高度 动画副作用，导致计算问题？ 旧的to: { height: show ? bounds.height : 0 },
    //这个height实际是一个对象的。  const { animationObj } = useSpring ；注意两个height不是一个东西。
    //这里useSpring({）的参数 实际是初始化用的，后面输出的对象height.value实际是动画编制的起始数值，而不是目标数值。
    //[问题！] 这里的useSpring（）没有处理内部DIV高度变化的副作用；打印机场景div高度会缩放调整，应该被缩小的<animated.div却没有被缩小。
    var height = useSpring({
        from: { height: 0 },
        to: { height: show ? bounds.height : 0 },
        immediate: prevShow !== null && prevShow === show
    }).height;
    //实际打印预览会捕获3次的render这里，Collapse-捕获height=，前面2次纸张缩放调整，缩小了，div就会变矮化，后面第三次却是屏幕的。
    //只好先预留大一点的高度。 有待改进。
    var dynamicHeight = (bounds.height > (height && height.value)) ? (bounds.height) : (height && height.value) > 0 ? (height && height.value) : undefined;
    return (jsx(React.Fragment, null, noAnimated ? (jsx("div", tslib_1.__assign({ id: id, css: {
            display: show ? undefined : 'none',
            overflow: "hidden",
        } }, other),
        jsx("div", { ref: ref, css: divStyle }, children)))
        :
            (jsx("div", { css: [
                    (_b = {
                            height: show ? dynamicHeight : 0
                        },
                        _b["@media not print"] = {
                            display: show ? undefined : 'none',
                        },
                        _b["@media print"] = {
                            overflow: "hidden",
                        },
                        _b),
                ] },
                jsx(animated.div, tslib_1.__assign({ id: id, style: { height: height }, css: {
                        willChange: "height, opacity",
                    } }, other),
                    jsx("div", { ref: ref, css: divStyle }, children))))));
};
Collapse.propTypes = {
    children: PropTypes.node,
    id: PropTypes.string.isRequired,
    show: PropTypes.bool.isRequired
};
export function usePrevious(value) {
    var ref = React.useRef(null);
    React.useEffect(function () {
        ref.current = value;
    }, [value]);
    return ref.current;
}
