/** @jsx jsx */
import { SerializedStyles } from "@emotion/core";
import * as React from "react";
import { Bounds } from "./Hooks/use-measure";
/**
 * Combobox context
 */
interface ComboBoxContextType {
    /** element refs */
    inputRef: React.RefObject<HTMLInputElement>;
    listRef: React.RefObject<HTMLElement>;
    /** list options */
    options: React.MutableRefObject<string[] | null>;
    makeHash: (i: string) => string;
    selected: string | null;
    expanded: boolean;
    /** event handlers */
    onInputChange: (e: React.ChangeEvent<HTMLInputElement>) => void;
    handleBlur: () => void;
    handleFocus: () => void;
    handleOptionSelect: (value: string) => void;
    onKeyDown: (e: React.KeyboardEvent) => void;
    /** popover positions */
    position: any;
    inputSize: Bounds;
    listId: string;
    query: string;
    autocomplete: boolean;
    topDivStyle: SerializedStyles | undefined;
}
export declare const ComboBoxContext: React.Context<ComboBoxContextType | null>;
/**
 * Context provider / manager
 */
export interface ComboBoxProps {
    onSelect?: (selected: string) => void;
    query: string;
    onQueryChange: (value: string) => void;
    autocomplete?: boolean;
    topDivStyle?: SerializedStyles;
}
export declare const ComboBox: React.FunctionComponent<ComboBoxProps>;
/**
 * Input element
 */
export interface ComboBoxInputProps extends React.HTMLAttributes<any> {
    "aria-label": string;
    component?: React.ReactType<any>;
    [key: string]: any;
}
export declare const ComboBoxInput: React.FunctionComponent<ComboBoxInputProps>;
/**
 * Popover container
 */
export interface ComboBoxListProps extends React.HTMLAttributes<HTMLUListElement> {
    autoHide?: boolean;
}
export declare const ComboBoxList: React.FunctionComponent<ComboBoxListProps>;
/**
 * Individual combo box options
 */
export interface ComboBoxOptionProps extends React.HTMLAttributes<HTMLLIElement> {
    value: string;
}
export declare const ComboBoxOption: React.FunctionComponent<ComboBoxOptionProps>;
/**
 * ComboBox Item text with highlighting
 */
export interface ComboBoxOptionTextProps {
    value: string;
}
export declare const ComboBoxOptionText: React.FunctionComponent<ComboBoxOptionTextProps>;
export {};
