import * as tslib_1 from "tslib";
/** @jsx jsx */
import { jsx } from "@emotion/core";
import * as React from "react";
import { usePositioner } from "./Hooks/use-positioner";
import { useUid } from "./Hooks/use-uid";
import { safeBind } from "./Hooks/compose-bind";
import { useTheme } from "./Theme/Providers";
import { Text } from "./Text";
import { Touchable } from "./Touchable";
import { useMeasure } from "./Hooks/use-measure";
import { Layer } from "./Layer";
import Highlighter from "react-highlight-words";
import { InputBase } from "./Form";
export var ComboBoxContext = React.createContext(null);
export var ComboBox = function (_a) {
    var children = _a.children, onSelect = _a.onSelect, _b = _a.autocomplete, autocomplete = _b === void 0 ? false : _b, query = _a.query, onQueryChange = _a.onQueryChange, topDivStyle = _a.topDivStyle;
    var inputRef = React.useRef(null);
    var listRef = React.useRef(null);
    var listId = "list" + useUid();
    var options = React.useRef([]);
    var position = usePositioner({
        modifiers: {
            flip: {
                enabled: false
            }
        }
    });
    var _c = React.useState(false), expanded = _c[0], setExpanded = _c[1];
    var _d = React.useState(null), selected = _d[0], setSelected = _d[1];
    var inputSize = useMeasure(inputRef);
    /**
     * Handle input change
     */
    var onInputChange = React.useCallback(function (e) {
        // potentially show popover
        setSelected(null);
        if (!expanded) {
            setExpanded(true);
        }
        onQueryChange(e.target.value);
    }, [expanded]);
    /**
     * Escape key pressed
     */
    var onEscape = React.useCallback(function () {
        setExpanded(false);
        setSelected(null);
    }, []);
    /**
     * Enter pressed or item clicked
     */
    var onItemSelect = React.useCallback(function () {
        // call the parent with the selected value?
        setExpanded(false);
        onSelect && onSelect(selected);
        setSelected(null);
    }, [selected]);
    /**
     * Get the currently active option index
     * */
    var getSelectedIndex = React.useCallback(function () {
        if (!selected)
            return -1;
        return options.current.indexOf(selected || "");
    }, [options, selected]);
    /**
     * Arrow up pressed
     */
    var onArrowUp = React.useCallback(function () {
        var opts = options.current;
        var i = getSelectedIndex();
        // on input? cycle to bottom
        if (i === -1) {
            setSelected(opts[opts.length - 1]);
            // select prev
        }
        else {
            setSelected(opts[i - 1]);
        }
    }, [getSelectedIndex]);
    /**
     * Arrow down pressed
     */
    var onArrowDown = React.useCallback(function () {
        var opts = options.current;
        var i = getSelectedIndex();
        // if last, cycle to first
        if (i + 1 === opts.length) {
            setSelected(opts[0]);
            // or next
        }
        else {
            setSelected(opts[i + 1]);
        }
    }, [getSelectedIndex, selected]);
    /**
     * Handle keydown events
     */
    var onKeyDown = React.useCallback(function (e) {
        switch (e.key) {
            case "ArrowUp":
                e.preventDefault();
                onArrowUp();
                break;
            case "ArrowDown":
                e.preventDefault();
                onArrowDown();
                break;
            case "Escape":
                e.preventDefault();
                onEscape();
                break;
            case "Enter":
                e.preventDefault();
                onItemSelect();
                break;
        }
    }, [onArrowDown, onArrowUp, onEscape, onItemSelect]);
    /**
     * Handle blur events
     */
    var handleBlur = React.useCallback(function () {
        requestAnimationFrame(function () {
            var focusedElement = document.activeElement;
            var list = listRef.current;
            if (focusedElement == inputRef.current || focusedElement == list) {
                // ignore
                return;
            }
            // ignore if our popover contains the focused element
            if (list && list.contains(focusedElement)) {
                return;
            }
            // hide popover
            setExpanded(false);
            setSelected(null);
        });
    }, []);
    /**
     * handle input focus
     */
    var handleFocus = React.useCallback(function () {
        setExpanded(true);
    }, []);
    /**
     * Handle item clicks
     */
    var handleOptionSelect = React.useCallback(function (value) {
        onSelect && onSelect(value);
        setExpanded(false);
        setSelected(null);
    }, []);
    /**
     * Make a unique hash for list + option
     */
    var makeHash = React.useCallback(function (i) {
        return listId + i;
    }, [listId]);
    return (jsx(ComboBoxContext.Provider, { value: {
            listId: listId,
            inputRef: inputRef,
            listRef: listRef,
            options: options,
            onInputChange: onInputChange,
            selected: selected,
            onKeyDown: onKeyDown,
            handleBlur: handleBlur,
            handleFocus: handleFocus,
            handleOptionSelect: handleOptionSelect,
            position: position,
            makeHash: makeHash,
            expanded: expanded,
            inputSize: inputSize.bounds,
            query: query,
            autocomplete: autocomplete,
            topDivStyle: topDivStyle,
        } }, children));
};
//分解配套的模式ComboBoxInput必需加到ComboBox底下的，ComboBox本身没有实际的div元素,靠context管理传递参数。
export var ComboBoxInput = function (_a) {
    var _b = _a.component, Component = _b === void 0 ? InputBase : _b, other = tslib_1.__rest(_a, ["component"]);
    var context = React.useContext(ComboBoxContext);
    var _c = React.useState(""), localValue = _c[0], setLocalValue = _c[1];
    if (!context) {
        throw new Error("ComboBoxInput must be wrapped in a ComboBox component");
    }
    //这里topDivStyle样式从上级组件ComboBox以共享同一个context模式传递的，算是第二次传递。
    var onKeyDown = context.onKeyDown, makeHash = context.makeHash, selected = context.selected, position = context.position, handleBlur = context.handleBlur, query = context.query, autocomplete = context.autocomplete, handleFocus = context.handleFocus, onInputChange = context.onInputChange, listId = context.listId, inputRef = context.inputRef, topDivStyle = context.topDivStyle;
    /** support autocomplete on selection */
    React.useEffect(function () {
        if (!autocomplete) {
            return;
        }
        if (selected) {
            setLocalValue(selected);
        }
        else {
            setLocalValue("");
        }
    }, [selected, autocomplete]);
    return (jsx(Component, tslib_1.__assign({ id: listId, onKeyDown: onKeyDown, onChange: onInputChange, onBlur: handleBlur, onFocus: handleFocus, "aria-controls": listId, autoComplete: "off", value: localValue || query, "aria-readonly": true, "aria-autocomplete": "list", role: "textbox", css: [
            {},
            topDivStyle
        ], "aria-activedescendant": selected ? makeHash(selected) : undefined }, safeBind({
        ref: inputRef
    }, {
        ref: position.target.ref
    }, other))));
};
export var ComboBoxList = function (_a) {
    var children = _a.children, _b = _a.autoHide, autoHide = _b === void 0 ? true : _b, other = tslib_1.__rest(_a, ["children", "autoHide"]);
    var context = React.useContext(ComboBoxContext);
    var theme = useTheme();
    if (!context) {
        throw new Error("ComboBoxInput must be wrapped in a ComboBox component");
    }
    var inputSize = context.inputSize, expanded = context.expanded, listId = context.listId, handleBlur = context.handleBlur, listRef = context.listRef, position = context.position, options = context.options;
    React.useLayoutEffect(function () {
        options.current = [];
        return function () {
            options.current = [];
        };
    });
    return (jsx(React.Fragment, null, (expanded || !autoHide) && (jsx(Layer, tslib_1.__assign({ tabIndex: -1, elevation: "sm", key: "1", style: position.popover.style, "data-placement": position.popover.placement, id: listId, role: "listbox", onBlur: handleBlur, className: "ComboBoxList", css: {
            overflow: "hidden",
            borderRadius: theme.radii.sm,
            outline: "none",
            width: inputSize.width +
                inputSize.left +
                (inputSize.right - inputSize.width) +
                "px",
            margin: 0,
            padding: 0,
            zIndex: theme.zIndices.overlay,
        } }, safeBind({
        ref: listRef
    }, {
        ref: position.popover.ref
    }, other)),
        children,
        jsx("div", { ref: position.arrow.ref, style: position.arrow.style })))));
};
export var ComboBoxOption = function (_a) {
    var value = _a.value, children = _a.children, other = tslib_1.__rest(_a, ["value", "children"]);
    var context = React.useContext(ComboBoxContext);
    var theme = useTheme();
    if (!context) {
        throw new Error("ComboBoxInput must be wrapped in a ComboBox component");
    }
    var makeHash = context.makeHash, handleOptionSelect = context.handleOptionSelect, options = context.options, selected = context.selected;
    React.useEffect(function () {
        if (options.current) {
            options.current.push(value);
        }
    });
    var isSelected = selected === value;
    var onClick = React.useCallback(function () {
        handleOptionSelect(value);
    }, [value]);
    return (jsx(Touchable, tslib_1.__assign({ tabIndex: -1, id: makeHash(value), role: "option", component: "li", onPress: onClick, "aria-selected": isSelected ? "true" : "false", css: {
            outline: "none",
            width: "100%",
            boxSizing: "border-box",
            display: "block",
            listStyleType: "none",
            margin: 0,
            padding: "0.5rem 0.75rem",
            cursor: "pointer",
            background: isSelected ? theme.colors.background.tint1 : "none",
            "&.Touchable--hover": {
                background: theme.colors.background.tint1
            },
            "&.Touchable--active": {
                background: theme.colors.background.tint2
            }
        } }, other), children || jsx(ComboBoxOptionText, { value: value })));
};
export var ComboBoxOptionText = function (_a) {
    var value = _a.value, other = tslib_1.__rest(_a, ["value"]);
    var context = React.useContext(ComboBoxContext);
    var theme = useTheme();
    if (!context) {
        throw new Error("ComboBoxInput must be wrapped in a ComboBox component");
    }
    var query = context.query;
    return (jsx(Text, tslib_1.__assign({ className: "ComboBoxOptionText", css: { fontWeight: theme.fontWeights.heading } }, other),
        jsx(Highlighter, { highlightStyle: {
                color: theme.colors.text.selected,
                background: "transparent"
            }, className: "ComboBoxOptionText__Highlighter", searchWords: [query], autoEscape: true, textToHighlight: value })));
};
