import * as tslib_1 from "tslib";
/** @jsx jsx */
import { jsx, css } from "@emotion/core";
import * as React from "react";
import { Text } from "./Text";
import VisuallyHidden from "@reach/visually-hidden";
import PropTypes from "prop-types";
import { alpha } from "./Theme/colors";
import { useUid } from "./Hooks/use-uid";
import { useTheme } from "./Theme/Providers";
import { getHeight, focusShadow } from "./Button";
import { IconAlertCircle, IconChevronDown } from "./Icons";
import { safeBind } from "./Hooks/compose-bind";
import { useMedia } from "use-media";
/*
自适应布局，容器父组件不应当设置width固定的px，否则内部组件元素{已经为屏幕宽度自适应适配的组件}都会被动拉伸宽度，失去了效果。
*/
var getInputSizes = function (theme) { return ({
    sm: css({
        fontSize: theme.fontSizes[0],
        padding: "0.25rem 0.5rem"
    }),
    md: css({
        fontSize: theme.fontSizes[1],
        padding: "0.5rem 0.75rem"
    }),
    lg: css({
        fontSize: theme.fontSizes[2],
        padding: "0.65rem 1rem"
    })
}); };
var InputGroupContext = React.createContext({
    uid: undefined,
    error: undefined
});
export var InputGroup = function (_a) {
    var id = _a.id, label = _a.label, children = _a.children, error = _a.error, helpText = _a.helpText, hideLabel = _a.hideLabel, labelStyle = _a.labelStyle, labelTextStyle = _a.labelTextStyle, other = tslib_1.__rest(_a, ["id", "label", "children", "error", "helpText", "hideLabel", "labelStyle", "labelTextStyle"]);
    var uid = useUid(id);
    var theme = useTheme();
    var isDark = theme.colors.mode === "dark";
    var danger = isDark
        ? theme.colors.intent.danger.light
        : theme.colors.intent.danger.base;
    return (jsx("div", tslib_1.__assign({ className: "InputGroup", css: {
            marginTop: theme.spaces.md,
            ":first-child": {
                marginTop: 0
            }
        } }, other),
        jsx(Label, { hide: hideLabel, htmlFor: uid, css: labelStyle, textStyle: labelTextStyle }, label),
        jsx(InputGroupContext.Provider, { value: {
                uid: uid,
                error: error
            } }, children),
        error && typeof error === "string" ? (jsx("div", { className: "InputGroup__error", css: {
                alignItems: "center",
                marginTop: theme.spaces.sm,
                display: "flex"
            } },
            jsx(IconAlertCircle, { size: "sm", color: danger }),
            jsx(Text, { css: {
                    display: "block",
                    marginLeft: theme.spaces.xs,
                    fontSize: theme.fontSizes[0],
                    color: danger
                } }, error))) : (error),
        helpText && (jsx(Text, { className: "InputGroup__help", css: {
                display: "block",
                marginTop: theme.spaces.xs,
                color: theme.colors.text.muted,
                fontSize: theme.fontSizes[0]
            }, variant: "body" }, helpText))));
};
InputGroup.propTypes = {
    label: PropTypes.string.isRequired,
    hideLabel: PropTypes.bool,
    helpText: PropTypes.string,
    error: PropTypes.oneOfType([PropTypes.string, PropTypes.node]),
    id: PropTypes.string,
    children: PropTypes.node
};
function shadowBorder(color, opacity) {
    return "0 0 0 2px transparent inset, 0 0 0 1px " + alpha(color, opacity) + " inset";
}
function getBaseStyles(theme) {
    var dark = theme.colors.mode === "dark";
    var baseStyles = css({
        display: "block",
        width: "100%",
        lineHeight: theme.lineHeights.body,
        color: theme.colors.text.default,
        backgroundColor: "transparent",
        backgroundImage: "none",
        backgroundClip: "padding-box",
        WebkitFontSmoothing: "antialiased",
        WebkitTapHighlightColor: "transparent",
        WebkitAppearance: "none",
        boxSizing: "border-box",
        touchAction: "manipulation",
        fontFamily: theme.fonts.base,
        border: "none",
        boxShadow: dark
            ? shadowBorder(theme.colors.palette.gray.lightest, 0.14)
            : shadowBorder(theme.colors.palette.gray.dark, 0.2),
        borderRadius: theme.radii.sm,
        transition: "background 0.25s cubic-bezier(0.35,0,0.25,1), border-color 0.15s cubic-bezier(0.35,0,0.25,1), box-shadow 0.15s cubic-bezier(0.35,0,0.25,1)",
        "::placeholder": {
            color: alpha(theme.colors.text.default, 0.45)
        },
        ":focus": {
            boxShadow: dark
                ? focusShadow(alpha(theme.colors.palette.blue.light, 0.5), alpha(theme.colors.palette.gray.dark, 0.4), alpha(theme.colors.palette.gray.light, 0.2))
                : focusShadow(alpha(theme.colors.palette.blue.dark, 0.1), alpha(theme.colors.palette.gray.dark, 0.2), alpha(theme.colors.palette.gray.dark, 0.05)),
            outline: "none"
        },
        ":disabled": {
            opacity: dark ? 0.4 : 0.8,
            background: theme.colors.background.tint1,
            cursor: "not-allowed",
            boxShadow: dark
                ? shadowBorder(theme.colors.palette.gray.lightest, 0.15)
                : shadowBorder(theme.colors.palette.gray.dark, 0.12)
        },
        ":active": {
            background: theme.colors.background.tint1
        }
    });
    return baseStyles;
}
function useActiveStyle() {
    var _a = React.useState(false), active = _a[0], setActive = _a[1];
    return {
        bind: {
            onTouchStart: function () { return setActive(true); },
            onTouchEnd: function () { return setActive(false); }
        },
        active: active
    };
}
function useSharedStyle() {
    var theme = useTheme();
    var errorStyles = {
        boxShadow: shadowBorder(theme.colors.intent.danger.base, 0.45)
    };
    var baseStyles = React.useMemo(function () { return getBaseStyles(theme); }, [theme]);
    var inputSizes = React.useMemo(function () { return getInputSizes(theme); }, [theme]);
    var activeBackground = css({ background: theme.colors.background.tint1 });
    return {
        baseStyles: baseStyles,
        inputSizes: inputSizes,
        activeBackground: activeBackground,
        errorStyles: errorStyles
    };
}
/**
 * Our basic Input element. Use this when building customized
 * forms. Otherwise, stick with InputGroup
 */
export var InputBase = React.forwardRef(function (_a, ref) {
    var autoComplete = _a.autoComplete, autoFocus = _a.autoFocus, _b = _a.inputSize, inputSize = _b === void 0 ? "md" : _b, topDivStyle = _a.topDivStyle, other = tslib_1.__rest(_a, ["autoComplete", "autoFocus", "inputSize", "topDivStyle"]);
    var _c = React.useContext(InputGroupContext), uid = _c.uid, error = _c.error;
    var _d = useActiveStyle(), bind = _d.bind, active = _d.active;
    var _e = useSharedStyle(), baseStyles = _e.baseStyles, inputSizes = _e.inputSizes, activeBackground = _e.activeBackground, errorStyles = _e.errorStyles;
    var height = getHeight(inputSize);
    return (jsx("input", tslib_1.__assign({ id: uid, className: "Input", autoComplete: autoComplete, autoFocus: autoFocus }, bind, { css: [
            baseStyles,
            inputSizes[inputSize],
            active && activeBackground,
            error && errorStyles,
            { height: height },
            topDivStyle
        ] }, safeBind({ ref: ref }, other))));
});
InputBase.propTypes = {
    inputSize: PropTypes.oneOf(["sm", "md", "lg"]),
    autoComplete: PropTypes.string,
    autoFocus: PropTypes.bool
};
//包裹一个div以便于控制宽度和对齐。
export var Input = React.forwardRef(function (_a, ref) {
    var autoComplete = _a.autoComplete, autoFocus = _a.autoFocus, _b = _a.inputSize, inputSize = _b === void 0 ? "md" : _b, _c = _a.fullWidth, fullWidth = _c === void 0 ? true : _c, topDivStyle = _a.topDivStyle, other = tslib_1.__rest(_a, ["autoComplete", "autoFocus", "inputSize", "fullWidth", "topDivStyle"]);
    var _d = React.useContext(InputGroupContext), uid = _d.uid, error = _d.error;
    var _e = useActiveStyle(), bind = _e.bind, active = _e.active;
    var _f = useSharedStyle(), baseStyles = _f.baseStyles, inputSizes = _f.inputSizes, activeBackground = _f.activeBackground, errorStyles = _f.errorStyles;
    var height = getHeight(inputSize);
    return (jsx("div", { css: [
            {
                textAlign: 'left',
                width: "100%"
            },
            topDivStyle
        ] },
        jsx("input", tslib_1.__assign({ id: uid, className: "Input", autoComplete: autoComplete, autoFocus: autoFocus }, bind, { css: [
                baseStyles,
                inputSizes[inputSize],
                active && activeBackground,
                error && errorStyles,
                { height: height },
                !fullWidth && {
                    width: 'unset',
                }
            ] }, safeBind({ ref: ref }, other)))));
});
/**
 * Textarea version of InputBase
 */
export var TextArea = function (_a) {
    var _b = _a.inputSize, inputSize = _b === void 0 ? "md" : _b, topDivStyle = _a.topDivStyle, other = tslib_1.__rest(_a, ["inputSize", "topDivStyle"]);
    var _c = useActiveStyle(), bind = _c.bind, active = _c.active;
    var _d = useSharedStyle(), baseStyles = _d.baseStyles, inputSizes = _d.inputSizes, activeBackground = _d.activeBackground, errorStyles = _d.errorStyles;
    var _e = React.useContext(InputGroupContext), uid = _e.uid, error = _e.error;
    return (jsx("textarea", tslib_1.__assign({ className: "TextArea", id: uid }, bind, { css: [
            baseStyles,
            inputSizes[inputSize],
            {
                overflow: "auto",
                resize: "vertical",
            },
            active && activeBackground,
            error && errorStyles,
            topDivStyle
        ] }, other)));
};
TextArea.propTypes = {
    inputSize: PropTypes.oneOf(["sm", "md", "lg"])
};
/**
 * A styled Label to go along with input elements
 */
export var Label = function (_a) {
    var children = _a.children, hide = _a.hide, textStyle = _a.textStyle, other = tslib_1.__rest(_a, ["children", "hide", "textStyle"]);
    var theme = useTheme();
    var child = (jsx("label", tslib_1.__assign({ className: "Label", css: {
            display: "inline-block",
            marginBottom: hide ? 0 : theme.spaces.sm
        } }, other),
        jsx(Text, { className: "Label__text", variant: "subtitle", css: textStyle }, children)));
    return hide ? jsx(VisuallyHidden, null, child) : child;
};
Label.propTypes = {
    hide: PropTypes.bool,
    children: PropTypes.node
};
/**
 * A styled select
 * 若触摸屏 不能支持multiple形态的的Select，只能单选。
 */
export var Select = function (_a) {
    var multiple = _a.multiple, _b = _a.inputSize, inputSize = _b === void 0 ? "md" : _b, divStyle = _a.divStyle, topDivStyle = _a.topDivStyle, other = tslib_1.__rest(_a, ["multiple", "inputSize", "divStyle", "topDivStyle"]);
    var theme = useTheme();
    var inputSizes = getInputSizes(theme);
    var _c = React.useContext(InputGroupContext), uid = _c.uid, error = _c.error;
    var selectSize = {
        sm: inputSizes.sm,
        md: inputSizes.md,
        lg: inputSizes.lg
    };
    var dark = theme.colors.mode === "dark";
    var height = getHeight(inputSize);
    //因需要Select组件的max-width；导致Select若放InputGroupLine下在宽松模式一行内显示Label和Select的场景，在Select组件头层div设置宽度将会使得flex无法对齐两个项目；所以再套入一个div。
    return (jsx("div", { css: [
            {
                textAlign: 'left'
            },
            topDivStyle
        ] },
        jsx("div", { className: "Select", css: [
                {
                    //这个position: "relative"因为其下级的position: "absolute"定位的小小图标所约束。所以只能relative或sticky其他position取值不行。
                    position: "relative",
                },
                divStyle,
            ] },
            jsx("select", tslib_1.__assign({ className: "Select__input", id: uid, css: [
                    selectSize[inputSize],
                    {
                        WebkitAppearance: "none",
                        display: "block",
                        width: "100%",
                        // lineHeight: theme.lineHeights.body,
                        height: multiple ? 'unset' : height,
                        color: theme.colors.text.default,
                        background: "transparent",
                        fontFamily: theme.fonts.base,
                        boxShadow: "0 0 0 2px transparent inset, 0 0 0 1px " + (dark
                            ? alpha(theme.colors.palette.gray.lightest, 0.14)
                            : alpha(theme.colors.palette.gray.dark, 0.2)) + " inset",
                        border: "none",
                        backgroundClip: "padding-box",
                        borderRadius: theme.radii.sm,
                        margin: 0,
                        ":disabled": {
                            ":disabled": {
                                opacity: dark ? 0.4 : 0.8,
                                background: theme.colors.background.tint1,
                                cursor: "not-allowed",
                                boxShadow: "0 0 0 2px transparent inset, 0 0 0 1px " + (dark
                                    ? alpha(theme.colors.palette.gray.lightest, 0.15)
                                    : alpha(theme.colors.palette.gray.dark, 0.12)) + " inset"
                            }
                        },
                        ":focus": {
                            borderColor: theme.colors.palette.blue.base,
                            boxShadow: dark
                                ? focusShadow(alpha(theme.colors.palette.blue.light, 0.5), alpha(theme.colors.palette.gray.dark, 0.4), alpha(theme.colors.palette.gray.light, 0.2))
                                : focusShadow(alpha(theme.colors.palette.blue.dark, 0.1), alpha(theme.colors.palette.gray.dark, 0.2), alpha(theme.colors.palette.gray.dark, 0.05)),
                            outline: 0
                        }
                    },
                    error && {
                        boxShadow: shadowBorder(theme.colors.intent.danger.base, 0.45)
                    }
                ], multiple: multiple }, other)),
            !multiple && (jsx(IconChevronDown, { className: "Select__icon", color: theme.colors.text.muted, css: {
                    position: "absolute",
                    top: "50%",
                    right: "0.75rem",
                    transform: "translateY(-50%)",
                    pointerEvents: "none"
                } })))));
};
Select.propTypes = {
    inputSize: PropTypes.oneOf(["sm", "md", "lg"]),
    multiple: PropTypes.bool
};
export var Check = function (_a) {
    var label = _a.label, id = _a.id, disabled = _a.disabled, topDivStyle = _a.topDivStyle, other = tslib_1.__rest(_a, ["label", "id", "disabled", "topDivStyle"]);
    var uid = useUid(id);
    var theme = useTheme();
    return (jsx("div", tslib_1.__assign({ className: "Check", css: [
            {
                //  textAlign: 'left',
                display: "flex",
                alignItems: "center",
            },
            topDivStyle
        ] }, other),
        jsx("input", tslib_1.__assign({ disabled: disabled, className: "Check__input", type: "checkbox", id: uid, css: [
                {
                    height: '2rem',
                    width: '2rem',
                    display: 'inline-flex',
                    paddingLeft: '1rem',
                },
            ] }, other)),
        jsx("label", { className: "Check__label", css: {
                opacity: disabled ? 0.6 : undefined,
                marginLeft: theme.spaces.xs
            }, htmlFor: uid },
            jsx(Text, null, label))));
};
Check.propTypes = {
    label: PropTypes.string.isRequired,
    id: PropTypes.string,
    disabled: PropTypes.bool
};
/**
 * A styled Label to go along with input elements
 * 若children有，就是有附带单位后缀串的模式；
 * 带单位后缀的说明，70%给输入框，后面30%给叙述单位字串{空格也算}
 * 输入的 单位说明 字符串 放在 <Text className="Suffix__text"  >
 */
export var SuffixInput = function (_a) {
    var children = _a.children, textStyle = _a.textStyle, inputSize = _a.inputSize, topDivStyle = _a.topDivStyle, other = tslib_1.__rest(_a, ["children", "textStyle", "inputSize", "topDivStyle"]);
    var theme = useTheme();
    return (jsx("div", { css: [
            {
                textAlign: 'left'
                //display: "inline-block",
            },
            topDivStyle
        ] },
        jsx(InputBase, tslib_1.__assign({ inputSize: inputSize, css: {
                display: "inline-block",
                width: children ? "70%" : '100%',
            } }, other)),
        jsx(Text, { className: "Suffix__text", variant: "subtitle", css: [
                {
                    display: children ? "inline-flex" : 'none',
                    paddingLeft: '0.2rem'
                },
                textStyle
            ] }, children)));
};
SuffixInput.propTypes = {
    children: PropTypes.node
};
/*
自适应屏幕flexBox布局：不要设置固定的width和min-width，可以设置max-width；根据屏幕宽策划1列2列还是更多列的并列，或是更高层次嵌套或隐藏或显示一小半边天区域。
不要对InputGroupLine的上一级div定义固定宽度，自适应和固定width: px只能二者选其一；宽度定了对小屏幕场景就有滚动条，而不是自适应缩小flexBox布局。
修改InputGroup排版模式; 并排模式，根据屏幕自适应。支持 2 个模式的布局安排结构。
*/
export var InputGroupLine = function (_a) {
    var id = _a.id, label = _a.label, children = _a.children, error = _a.error, helpText = _a.helpText, hideLabel = _a.hideLabel, labelStyle = _a.labelStyle, labelTextStyle = _a.labelTextStyle, lineStyle = _a.lineStyle, _b = _a.switchPx, switchPx = _b === void 0 ? 360 : _b, other = tslib_1.__rest(_a, ["id", "label", "children", "error", "helpText", "hideLabel", "labelStyle", "labelTextStyle", "lineStyle", "switchPx"]);
    var uid = useUid(id);
    var theme = useTheme();
    var isDark = theme.colors.mode === "dark";
    var danger = isDark
        ? theme.colors.intent.danger.light
        : theme.colors.intent.danger.base;
    //根据外部程序制定的px数，来决定用哪一个模式布局。紧凑的是2行显示；宽松的是并列在同一行。
    var fitable = useMedia({ minWidth: switchPx + "px" });
    var labelDivCss = css({
        flex: 1,
        paddingRight: '0.8rem'
    }, labelTextStyle);
    //InputGroupLine包裹的下层的顶级组件的样式修改：下层顶级元素的display: block还算兼容可用; 但width: 100%影响较大。
    var childNodeVar = (jsx(InputGroupContext.Provider, { value: {
            uid: uid,
            error: error
        } }, React.cloneElement(children, {
        topDivStyle: { flex: '1 1 60%' },
    })));
    return (jsx("div", tslib_1.__assign({ className: "InputGroup", css: {
            marginTop: theme.spaces.md,
            ":first-child": {
                marginTop: 0
            },
            textAlign: 'center'
        } }, other),
        jsx("div", { css: [
                {
                    alignItems: "center",
                    justifyContent: "space-around",
                    display: "flex",
                    flexWrap: 'wrap',
                    maxWidth: '950px',
                    margin: '0 auto',
                    paddingRight: fitable ? '0.5rem' : 'unset',
                },
                lineStyle
            ] },
            jsx(Label, { hide: hideLabel, htmlFor: uid, textStyle: labelDivCss, css: [
                    {
                        display: "inline-flex",
                        textAlign: fitable ? "right" : "left",
                        flex: '1 1 40%',
                    },
                    labelStyle
                ] }, label),
            fitable && childNodeVar),
        !fitable && childNodeVar,
        error && typeof error === "string" ? (jsx("div", { className: "InputGroup__error", css: {
                alignItems: "center",
                marginTop: theme.spaces.sm,
                display: "flex",
                justifyContent: 'center'
            } },
            jsx(IconAlertCircle, { size: "sm", color: danger }),
            jsx(Text, { css: {
                    display: "block",
                    marginLeft: theme.spaces.xs,
                    fontSize: theme.fontSizes[0],
                    color: danger
                } }, error))) : (error),
        helpText && (jsx(Text, { className: "InputGroup__help", css: {
                display: "inline-flex",
                marginTop: theme.spaces.xs,
                color: theme.colors.text.muted,
                fontSize: theme.fontSizes[0]
            }, variant: "body" }, helpText))));
};
InputGroupLine.propTypes = {
    label: PropTypes.string.isRequired,
    hideLabel: PropTypes.bool,
    helpText: PropTypes.string,
    error: PropTypes.oneOfType([PropTypes.string, PropTypes.node]),
    id: PropTypes.string,
    switchPx: PropTypes.number,
    children: PropTypes.node
};
